
function [] = TE_multi(create, plot_p, plot_s, varargin)

% this function can be used both to create TE calculations and plot the results. 
% Parameters can be set in the file itself (next section) or via the
% varargin.

% INPUT PARAMETERS:   

 %      [create]:   1, if TE calculations are to be performed
 %      [plot_p]:   1, if p-values are to be plotted
 %      [plot_s]:   1, if significance values are to be plotted
 %      [varargin]: set parameters for calculation/evaluation.  
 %                  All of these are optional since values have to be given in the next paragraph of this file.  
 %                  Values from [varargin] will overwrite those read in this file.
 %
 %                  Usage: TE_multi(create, display, 'par1', value1, 'par2', value2, ...)  
 %  
 %                  The following parameters can be set via varargin: 
 %                  nx,ny,nT,nS,g,u,cpl,eta_x    (multiple values possible)
 %                  dt,eta_y,n_runs              (only one value will be processed)
 % 
 %                  Note: values for cpl must be given in a cell array of strings.
 %
 %                  Example: TE_multi(1,0,0,'cpl',{'lin'},'nruns',20,'nx',[0.1 0.3 0.5])  
 %                     simulates and calculates TE for 20 runs each with linear coupling 
 %                     for nx values of 0.1, 0.3 and 0.5 
 %                  
 %                  The data will be stored in and/or read from: 
 %                  .../SimData/          where ... stands for the working directory 
 %                  .../SimData/[sdir]/   if you specify [sdir] as a string in varargin.
 %
 %                  If [create] is 'true' and [sdir] does not exist, it will be created.
 %                  Warning: Don't use a parameter's name as name for [sdir]!  
 %
 %                  Example: TE_multi(1,0,0,'sdir','test1') - stores data in .../SimData/test1
 %                           TE_multi(0,0,1,'sdir','test2') - reads data from .../SimData/test2, if it exists.
 %                           TE_multi(1,0,0,'sdir','nx')    - not a good idea!
 %
 %                  
 %                  For plotting significance values as a function of one of the parameters, specify 
 %                  this parameter by putting 'plotthis' in front of its specification.    
 %
 %                  Example: TE_multi(0,0,1,'plotthis','nx',[0 0.5 1 1.5], ...)
 %                  - plots significance values as a function of nx for values nx = 0, 0.5, 1 and 1.5.
 %                  As of now, parameters that can be used are: nx, nS, nT, u
 %
 %                  You can specify the number of rows and columns of plots in your figure with 'plotdim'.
 %                  Make sure #rows x #cols >= 2*the number of parameter combinations. 
 %                  
 %                  Example: TE_multi(0,0,1,'plotdim',[3 4],...) - creates a figure with 3x4 plots.
 %                  
 %                   %
 %                  Parameters are optimized with cao criterion unless you include a string 'ragwitz' in varargin. %
 %                  To use M. Lindner's code for quad. coupled AR series simulation, include a string 'Michael' in varargin.
 




%% set parameters

par.n_runs   = 100;                  % # runs per parameter combination
par.eta_y    = 0.1;                 % strength of the innovation term for the AR process in the y series  
par.dt       = 20;                  % coupling delay in samples
par.order    = 10;                  % AR order  
par.fsample  = 1000;                % sampling frequency
par.mode     = 'Felix';             % mode of generation of AR series. 'Felix' or 'Michael' (Lindner).
par.opt_meth = 'cao';               % method to optimize parameters: 'cao' or 'ragwitz'
par.sizeNei  = 4;                   % value for neighbourhood search

par.path = strcat(pwd,'/SimData/');
par.prelim = 0;
par.srun = 1;                       % running # of first run: if you want to resume calculations on a par. set 
cfg0 = par;

% the following parameteres can have multiple values, creating multiple parameter sets:

par.cpl     = {'lin','quad','tresh','mod'};     % coupling type: 'lin', 'quad', 'tresh', 'mod'
par.u       = [21];                       % prediction time
par.eta_x   = [0.1];                      % strength of the innovation term for the AR process in the x series
par.nx      = [0.1];                      % noise on x
par.ny      = [0.1];                      % noise on y
par.nT      = [40];                       % # trials
par.nS      = [3000];                     % # samples per trial
par.g       = [1];                        % coupling strength

%% plotting options

plotpar.design = 'altn';            % for p-values plot: 
                                    % 'altn': x->y and y->x values alternating
                                    % 'blck': plot all x->y values first, then all y->x values
                            
plotpar.n_rows   = 2;               % # of rows
plotpar.n_cols   = 4;               % # of columns, 
                                    % make sure: n_rows*n_cols >= 2*(# of parameter sets) !!!
                                    

plotpar.plot_pval = plot_p;          % plot p-values (or not)                            
 plotpar.p_means = 0;            % show means of p-values (not very useful)

plotpar.plot_sign = plot_s;          % plot results of significance tests  
 plotpar.plot_shift= 1;          % include results of shift tests 

if  plot_p || plot_s
    display = 1;
else 
    display = 0;
end
 
plotpar.suptext = 1;               

%% read varargin                       
                                    
n_varg = size(varargin,2);

if n_varg > 1                                   %can be improved with fieldnames(par)
  for i =  1:n_varg
      if strcmp(varargin{i},'nx')
        par.nx = varargin{i+1};
      end
      if strcmp(varargin{i},'ny')
        par.ny = varargin{i+1};
      end
      if strcmp(varargin{i},'nT')
        par.nT = varargin{i+1};
      end
      if strcmp(varargin{i},'nS')
        par.nS = varargin{i+1};
      end
      if strcmp(varargin{i},'g')
        par.g = varargin{i+1};
      end
      if strcmp(varargin{i},'u')
        par.u = varargin{i+1};
      end      
      if strcmp(varargin{i},'cpl')
        par.cpl = varargin{i+1};             
      end
      if strcmp(varargin{i},'dt')
        par.dt = varargin{i+1};
      end   
      if strcmp(varargin{i},'eta_x')         
        par.eta_x    = varargin{i+1};        
      end
      if strcmp(varargin{i},'eta_y')
        par.eta_y    = varargin{i+1};
      end
      if strcmp(varargin{i},'n_runs')
        par.n_runs = varargin{i+1};
      end
      if strcmp(varargin{i},'plotthis')
        plotpar.plotthis = varargin{i+1};
      end
      if strcmp(varargin{i},'prelim')
        par.prelim = 1;
      end      
      if strcmp(varargin{i},'srun')
        par.srun = varargin{i+1};
      end  
      if strcmp(varargin{i},'ragwitz')
        par.opt_meth = 'ragwitz';
      end
      if strcmp(varargin{i},'sizeNei')
        par.sizeNei  = varargin{i+1};
      end
      if strcmp(varargin{i},'Michael')
        par.mode     = 'Michael';
      end
      if strcmp(varargin{i},'plotdim')
        plotpar.n_rows = varargin{i+1}(1);        
        plotpar.n_cols = varargin{i+1}(2);
      end      
      if strcmp(varargin{i},'sdir')
        sdir = varargin{i+1};        
        newdir = strcat('SimData/',sdir)
        if ~isdir(newdir)
            if create
                mkdir(newdir);
            else
                error('Directory not found!')
            end
        end
        par.path  = strcat(par.path,sdir,'/');    
      end
      
  end  
end  
cfg0=par  
pause(5);
%% create XY-series and calculate TE
if (create)
    for a1 = 1 : size(par.cpl,2)
    for a2 = 1 : size(par.u ,2)
    for a3 = 1 : size(par.nx,2)
    for a4 = 1 : size(par.ny,2)
    for a5 = 1 : size(par.nT,2)
    for a6 = 1 : size(par.nS,2)
    for a7 = 1 : size(par.g ,2)
    for a8 = 1 : size(par.eta_x ,2)
                                    
    cfg1 = cfg0;

    cfg1.coupling =  par.cpl{a1};             % type of x->y coupling, "lin' or 'quad' or 'tresh'
    cfg1.u        =  par.u (a2);              % prediction time
    cfg1.x_noise  =  par.nx(a3);              % strength of the noise signal that is laid on top of the x series
    cfg1.y_noise  =  par.ny(a4);              % strength of the noise signal that is laid on top of the y series
    cfg1.nTrials  =  par.nT(a5);              % number of trials
    cfg1.nSamples =  par.nS(a6);              % samples per trial
    cfg1.gamma    =  par.g (a7);              % coupling strength
    cfg1.ex       =  par.eta_x(a8);           % innovation strength for x
    cfg1.toi = cfg1.nSamples/cfg1.fsample;    % time of interest

    TE_coupling(cfg1,par);                    %% simulate data and calculate TE

    end
    end
    end
    end
    end
    end
    end
    end

    
msg_out = strcat('Finished calculation at: ', datestr(now));
disp(msg_out);
end


%% read and plot p-values for TE
if (display)
    
    a=0;
    clear values;
  fullsize = size(par.cpl,2)*size(par.u,2)*size(par.nx,2)*size(par.ny,2)*size(par.nT,2)*size(par.nS,2)*size(par.g,2)*size(par.eta_x,2);
%   for f = 1:fullsize
%       values(f).c = f;
%   end
    
    for a1 = 1 : size(par.cpl,2)
    for a2 = 1 : size(par.u  ,2)
    for a3 = 1 : size(par.nx ,2)
    for a4 = 1 : size(par.ny ,2)
    for a5 = 1 : size(par.nT ,2)
    for a6 = 1 : size(par.nS ,2)
    for a7 = 1 : size(par.g  ,2)
    for a8 = 1 : size(par.eta_x,2)
        
        a = a+1;
        str.cpl = par.cpl{a1};
        str.u   = sprintf('%02d' ,par.u (a2));
        par.u(a2);
        if mod(par.u(a2),1) ~= 0
            str.u  = sprintf('%02.1f',par.u(a2));
        end
        str.nx  = sprintf('%1.1f',par.nx(a3));
        str.ny  = sprintf('%1.1f',par.ny(a4));
        str.nT  = sprintf('%02d' ,par.nT(a5));
        str.nS  = sprintf('%02d' ,par.nS(a6));
        str.g   = sprintf('%1.1f',par.g (a7));
        str.ex  = sprintf('%1.2f',par.eta_x(a8));
        
        par.a1 = a1;
        par.a2 = a2;
        par.a3 = a3;
        par.a4 = a4;
        par.a5 = a5;
        par.a6 = a6;
        par.a7 = a7;
        par.a8 = a8;
        par.toi = par.nS(a6)/par.fsample;
        values(a) = TE_stat(par,str);                  % read and label values
        fullsize
        a
        
    end  
    end
    end
    end
    end
    end
    end
    end     
        
%    display(values.mean_dim)
%    display(values.var_dim)
%    display(values.mean_tau)
%    display(values.var_tau)
make_fig(par, values, plotpar);                         % create plot
end

%%
% disp('Significance:')
% values.sign
% disp('Significance after shift test: ')
% values.sign2
% disp('Instant Mixing:')
% values.instmix







